/*
 * Sixth Data. Author: Svjatoslav Agejenko. 
 * This project is released under Creative Commons Zero (CC0) license.
 *
*/

package eu.svjatoslav.sixth.data.store.file;

public class VacuumContext {
    private final FileDataStore vacuumContext;
    private final int targetFragmentationPercent;
    long analyzedBytesCount = 0;
    long freeBytesCount = 0;
    long lastRecordStartAddress;
    long defragmentationStartAddress;

    public VacuumContext(final FileDataStore fileDataStore,
                         final long currentFreeAddress, final int targetFragmentationPercent) {

        vacuumContext = fileDataStore;
        lastRecordStartAddress = currentFreeAddress;
        defragmentationStartAddress = currentFreeAddress;
        this.targetFragmentationPercent = targetFragmentationPercent;
    }

    public void analyzeEntry(final EntryRecord record) {
        final long bytesSinceLastEntry = lastRecordStartAddress
                - record.location;

        final long freeSpace = bytesSinceLastEntry - record.length;
        freeBytesCount += freeSpace;
        analyzedBytesCount += freeSpace;

        checkDefragmentationPointer(record.location + record.length);

        analyzedBytesCount += record.length;
        lastRecordStartAddress = record.location;
    }

    public void analyzeStartOfDataArea() {
        freeBytesCount += lastRecordStartAddress
                - vacuumContext.metaData.getEntriesStorageAreaStart();

        checkDefragmentationPointer(vacuumContext.metaData
                .getEntriesStorageAreaStart());
    }

    public void checkDefragmentationPointer(
            final long defragmentationStartAddress) {

        if (analyzedBytesCount == 0) {
            this.defragmentationStartAddress = defragmentationStartAddress;
            return;
        }

        final int fragmentationPercentage = (int) ((100L * freeBytesCount) / analyzedBytesCount);

        if (fragmentationPercentage >= targetFragmentationPercent)
            this.defragmentationStartAddress = defragmentationStartAddress;
    }
}
